'''Add DRF SCPI to the current set of DRF SCPI.
'''
from .drv_scpi_iface import DrvScpiHandlerC, DrvScpiErrorC
from .drv_scpi_cmd import DrvScipiCmdTypeE, DrvScpiCmdDataC, DrvScpiDeviceC
from .drv_scpi_node import DrvScpiNodeC

__all__ = [
    'DrvScpiHandlerC',
    'DrvScpiErrorC',
    'DrvScipiCmdTypeE',
    'DrvScpiCmdDataC',
    'DrvScpiDeviceC',
    'DrvScpiNodeC'
]



#!/usr/bin/python3
'''
This module will manage SCPI messages and channels
in order to configure channels and send/received messages.
'''
#######################        MANDATORY IMPORTS         #######################
from __future__ import annotations

#######################         GENERIC IMPORTS          #######################

#######################       THIRD PARTY IMPORTS        #######################
from threading import Event

#######################      SYSTEM ABSTRACTION IMPORTS  #######################
import system_logger_tool as sys_log
if __name__ == "__main__":
    cycler_logger = sys_log.SysLogLoggerC()
log = sys_log.sys_log_logger_get_module_logger(__name__)

#######################          PROJECT IMPORTS         #######################
from system_shared_tool import SysShdChanC

#######################          MODULE IMPORTS          #######################
from .drv_scpi_iface import DrvScpiHandlerC
from .drv_scpi_cmd import DrvScipiCmdTypeE, DrvScpiCmdDataC

#######################              ENUMS               #######################

#######################             CLASSES              #######################
class DrvScpiNodeC:
    '''Returns a removable version of the DRV command.
    '''
    def __init__(self, tx_buffer: SysShdChanC, working_flag: Event):
        self.__used_dev: dict = {}
        self.working_flag: Event = working_flag
        self.tx_buffer: SysShdChanC = tx_buffer


    def __apply_command(self, cmd: DrvScpiCmdDataC):
        '''TODO: Poner titulo.
        Args:
            cmd (DrvScpiCmdDataC): [description]
        Returns:
            - None
        Raises:
            - None
        '''
        #Add device
        if cmd.payload is DrvScpiHandlerC:
            if cmd.data_type is DrvScipiCmdTypeE.ADD_DEV:
                if self.__used_dev.get(cmd.port) is None:
                    self.__used_dev[cmd.port] = cmd.payload
                else:
                    log.error("The port is already configured")
            else:
                log.error("Can`t apply command. \
                        Error in command format, check command type and payload type")

        #Send or receive data
        elif cmd.payload is str:
            message: str = cmd.payload # type: ignore
            scpi: DrvScpiHandlerC = self.__used_dev[cmd.port]
            if cmd.data_type is DrvScipiCmdTypeE.READ:
                scpi.receive_msg()
            elif cmd.data_type is DrvScipiCmdTypeE.WRITE:
                scpi.send_msg(message)
            elif cmd.data_type is DrvScipiCmdTypeE.WRITE_READ:
                scpi.send_and_read(message)
            else:
                log.error("Can`t apply command. \
                        Error in command format, check command type and payload type")


    def run(self) -> None:
        '''TODO: Poner titulo.
        Args:
            - None
        Returns:
            - None
        Raises:
            - None
        '''
        log.critical("Start running process")
        while self.working_flag.isSet():
            try:
                #Tx es salida, transmision. TODO: Falta recibir
                if not self.tx_buffer.is_empty():
                    command : DrvScpiCmdDataC = self.tx_buffer.receive_data() # type: ignore
                    self.__apply_command(command)
            except Exception as err:
                log.error(f"Error while receiving CAN message\n{err}")
        self.stop()


    def stop(self) -> None:
        '''TODO: Poner titulo.
        Args:
            - None
        Returns:
            - None
        Raises:
            - None
        '''
        log.critical("Stopping SCPI thread.")
        self.working_flag.clear()



#!/usr/bin/python3
"""
Driver for SCPI devices.
"""
#######################        MANDATORY IMPORTS         #######################
import re

#######################         GENERIC IMPORTS          #######################
from typing import List
from serial import EIGHTBITS, PARITY_NONE, STOPBITS_ONE, Serial

#######################       THIRD PARTY IMPORTS        #######################

#######################      SYSTEM ABSTRACTION IMPORTS  #######################
import system_logger_tool as sys_log

if __name__ == "__main__":
    cycler_logger = sys_log.SysLogLoggerC()
log = sys_log.sys_log_logger_get_module_logger(__name__)

#######################          PROJECT IMPORTS         #######################


#######################          MODULE IMPORTS          #######################


#######################              ENUMS               #######################
class _DefaultSerialParamsC:
    "Communication constants"
    _BAUD_RATE = 115200
    _BYTESIZE = EIGHTBITS
    _PARITY = PARITY_NONE
    _STOP_BITS = STOPBITS_ONE
    _READ_TIMEOUT = 0.5
    _WRITE_TIMEOUT = 0.5
    _INTER_BYTE_TIMEOUT = 0.5
    _MAX_LEN_IN_BYTES = 21


#######################             CLASSES              #######################
class DrvScpiErrorC(Exception):
    """Error class for SCPI driver."""

    def __init__(self, message: str, error_code: int) -> None:
        self.message = message
        self.error_code = error_code

    def __str__(self) -> str:
        return f"Error: {self.message} \t Code: {self.error_code}"


class DrvScpiHandlerC:
    """Driver for SCPI devices."""

    #pylint: disable=too-many-arguments
    def __init__(self, port: str, separator: str,
                baudrate: int = _DefaultSerialParamsC._BAUD_RATE,
                bytesize=_DefaultSerialParamsC._BYTESIZE,
                parity=_DefaultSerialParamsC._PARITY,
            stopbits=_DefaultSerialParamsC._STOP_BITS,
            timeout=_DefaultSerialParamsC._READ_TIMEOUT,
            write_timeout=_DefaultSerialParamsC._WRITE_TIMEOUT,
            inter_byte_timeout=_DefaultSerialParamsC._INTER_BYTE_TIMEOUT,
            ) -> None:

        self.__serial: Serial = Serial(port=port, baudrate=baudrate,
                                       bytesize=bytesize, parity=parity,
                                       stopbits=stopbits, timeout=timeout,
                                       write_timeout=write_timeout,
                                       inter_byte_timeout=inter_byte_timeout)
        self.__separator: str = separator

    def decode_numbers(self, data: str) -> float:
        """Decode bytes to integers.
        Args:
            - data (str): Value to convert to float.
        Returns:
            - msg_decode (float): Value decoded.
        Raises:
            - DrvScpiErrorC: Error decoding data.
        """
        result: list = re.findall(r"-?\d*\.?\d+", data)
        if len(result) < 2:
            msg_decode = float(result[0])
        elif len(result) == 2:
            msg_decode = float(result[0]) * 10 ** int(result[1])
        else:
            raise DrvScpiErrorC(message="Error decoding data", error_code=1)
        return msg_decode

    def decode_and_split(self, data: bytes) -> List[str]:
        """Decode str to integers and split the data.
        Args:
            data (bytes): Value to decode and split.
        Returns:
            msg_decode (List[str]): Message decoded and splited.
        Raises:
            - None
        """
        data_dec = data.decode("utf-8")
        msg_decode = data_dec.split(f"{self.__separator}")
        return msg_decode

    def read_device_info(self) -> List[str]:
        """Reads the list of device information.
        Args:
            - None
        Returns:
            - msg (List[str]): List of device information.
        Raises:
            - DrvScpiErrorC: Error decoding data.
        """
        msg = self.send_and_read("*IDN?")
        if len(msg) > 0:
            return msg
        raise DrvScpiErrorC(message="Error reading device information",
                            error_code=2)

    def send_msg(self, msg: str) -> None:
        """Send a message to the serial device.
        Args:
            - msg (str): Message to send.
        Returns:
            - None
        Raises:
            - None
        """
        msg = msg + self.__separator
        self.__serial.write(bytes(msg.encode("utf-8")))

    def receive_msg(self) -> List[str]:
        """
        Read until an separator is found, the size is exceeded or until timeout occurs.
        Args:
            - None
        Returns:
            - msg_decoded (List[str]): Received message of the device.
        Raises:
            - None
        """
        msg = self.__serial.readline()
        msg_decoded = self.decode_and_split(msg)
        return msg_decoded

    def send_and_read(self, msg: str) -> List[str]:
        """Send a message to the serial device and read the response.
        Args:
            - msg (str): Message to send.
        Returns:
            - msg (List[str]): Received message.
        Raises:
            - None
        """
        self.send_msg(msg)
        response: list = self.receive_msg()
        return response

    def close(self) -> None:
        """Close the serial connection.
        Args:
            - None
        Returns:
            - None
        Raises:
            - None
        """
        self.__serial.close()



#!/usr/bin/python3
'''
Driver for communication with SCPI devices.
'''
#######################        MANDATORY IMPORTS         #######################
from __future__ import annotations

#######################         GENERIC IMPORTS          #######################
from enum import Enum

#######################       THIRD PARTY IMPORTS        #######################

#######################      SYSTEM ABSTRACTION IMPORTS  #######################
import system_logger_tool as sys_log
if __name__ == "__main__":
    cycler_logger = sys_log.SysLogLoggerC()
log = sys_log.sys_log_logger_get_module_logger(__name__)

#######################          PROJECT IMPORTS         #######################
from system_shared_tool import SysShdChanC

#######################          MODULE IMPORTS          #######################
from .drv_scpi_iface import DrvScpiHandlerC

#######################              ENUMS               #######################
class DrvScipiCmdTypeE(Enum):
    '''Types of commands to be sent to the device.'''
    WRITE = 0
    READ = 1
    WRITE_READ = 2
    ADD_DEV = 3

#######################             CLASSES              #######################
class DrvScpiCmdDataC:
    '''
    Hold the data to be sent to the device.
    '''
    def __init__(self, data_type: DrvScipiCmdTypeE, port: str, payload: str|DrvScpiHandlerC):
        self.data_type: DrvScipiCmdTypeE = data_type
        self.port: str = port
        self.payload: str|DrvScpiHandlerC = payload


class DrvScpiDeviceC:
    '''
    Principal class of the driver.
    '''
    def __init__(self, port: str, payload: DrvScpiHandlerC, chan: SysShdChanC):
        self.port : str= port
        self.payload : DrvScpiHandlerC = payload
        self.chan : SysShdChanC = chan
